#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# notes2db.py
#
# Copyright (C) 2010 Gregorio Robles <grex@gsyc.urjc.es>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import sqlite3

paperList = []
paper = {}

f = open('notes.txt', 'r')

while 1:
    line = f.readline()
#    print line

    if not line:
        break
    
    if line[:3] == "===":
#        print paper
        if paper:
            paperList.append(paper)
        paper = {}

    elif line[:4] == "MSR:":
        paper['Venue'] = line[5:-1]
        
    elif line[:6] == "Title:":
        paper['Title'] = line[7:-1]

    elif line[:7] == "Length:":
        paper['Length'] = line[8:-1]

    elif line[:5] == "Type:":
        paper['Type'] = line[6:-1]

    elif line[:12] == "Public data:":
        paper['Public_data'] = line[13:-1]
        paper['Public_data_short'] = line[13]

    elif line[:14] == "Public source:":
        try:
            paper['publicSourceList'].append(line[15:-1])
        except KeyError:
            paper['publicSourceList'] = [line[15:-1]]
            
    elif line[:16] == "Working dataset:":
        paper['Working_dataset'] = line[17:-1]
        paper['Working_dataset_short'] = line[17]

    elif line[:13] == "Tool/scripts:":
        paper['Tools_scripts'] = line[14:-1]
        paper['Tools_scripts_short'] = line[14]

    else:
        print "ERROR", line

print "Some stats:"
print ""
print "Number of papers:", len(paperList)

summaryShort = ['Public_data_short', 'Working_dataset_short', 'Tools_scripts_short']

for summary in summaryShort:
    accumulate = ""
    for paper in paperList:
        try:
            accumulate += paper[summary]
        except KeyError:
            # Usually this is because it's a non-experimental paper
            accumulate += "/"
    print summary
    print "Y: ", accumulate.count('Y')
    print "N: ", accumulate.count('N')
    print "P: ", accumulate.count('P')
    print "/: ", accumulate.count('/')


# Let's put everything into DB

# CREATE DB tables
create_msr_table_query = "CREATE TABLE msr (paper_id INTEGER PRIMARY KEY, year INTEGER, title TEXT, length TEXT, type TEXT, Public_data TEXT, Public_data_short TEXT, Working_dataset TEXT, Working_dataset_short TEXT, Tools_scripts TEXT, Tools_scripts_short TEXT);"
create_tools_table_query = "CREATE TABLE tools (paper_id, source);"

conn = sqlite3.connect('msr.db')
c = conn.cursor()
c.execute(create_msr_table_query)
c.execute(create_tools_table_query)

# Introducing papers into database
for paper in paperList:

    if paper['Type'] == "Non-experimental":
        c.execute('INSERT INTO msr VALUES (null,?,?,?,?,?,?,?,?,?,?)', (paper['Venue'], paper['Title'], paper['Length'], paper['Type'], '', '', '', '', '', ''))
    else:
        c.execute('INSERT INTO msr VALUES (null,?,?,?,?,?,?,?,?,?,?)', (paper['Venue'], paper['Title'], paper['Length'], paper['Type'], paper['Public_data'], paper['Public_data_short'], paper['Working_dataset'], paper['Working_dataset_short'], paper['Tools_scripts'], paper['Tools_scripts_short']))

        # If the paper studies a public source, let's put them into the DB
        if 'publicSourceList' in paper:
            paper_id = c.lastrowid
            for source in paper['publicSourceList']:
                c.execute('INSERT INTO tools VALUES (?,?)', (paper_id, source))

    conn.commit()

# Let's get the study

queryDict = {
    'Number of papers submitted to MSR per year': 'SELECT year, COUNT(*) FROM msr GROUP BY year ORDER BY year;',
    'Number of papers by length and year submitted to MSR': 'SELECT year, length, COUNT(*) FROM msr GROUP BY length, year ORDER BY year;',
    
    'Number of papers by length, year and public availability of the data': 'SELECT year, length, Public_data_short, COUNT(*) FROM msr GROUP BY length, year, Public_data_short ORDER BY year;',
    'Number of papers by length and year and availability of the working dataset': 'SELECT year, length, Working_dataset_short, COUNT(*) FROM msr GROUP BY length, year, Working_dataset_short ORDER BY year;',
    'Number of papers by length and year and availability of tools': 'SELECT year, length, Tools_scripts_short, COUNT(*) FROM msr GROUP BY length, year, Tools_scripts_short ORDER BY year;',

    'Number of papers with data sources, working dataset and tools available': "SELECT COUNT(*) AS sum FROM msr WHERE Public_data_short='Y' AND Working_dataset_short == 'Y' AND Tools_Scripts_short=='Y';",
    '': "SELECT COUNT(*) AS sum FROM msr WHERE Public_data_short='Y' AND Working_dataset_short == 'N' AND Tools_Scripts_short=='Y';",
    '': "SELECT COUNT(*) AS sum FROM msr WHERE Public_data_short='Y' AND Working_dataset_short == 'N' AND Tools_Scripts_short=='N';",
    '': "SELECT COUNT(*) AS sum FROM msr WHERE Public_data_short='N' AND Working_dataset_short == 'N' AND Tools_Scripts_short=='N';",
    '': "SELECT COUNT(*) AS sum FROM msr WHERE Public_data_short='N' AND Working_dataset_short == 'N' AND Tools_Scripts_short=='Y';"
    }

for value in queryDict:
    c.execute(queryDict[value])
    print 
    print value
    for row in c:
        print row


queryDict = {
    'Top 20 most studied sources': "SELECT COUNT(*) AS count, source FROM tools GROUP BY source ORDER BY count DESC LIMIT 20;"
    }
